@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# __coconut_hash__ = 0x9aae9544
+# __coconut_hash__ = 0xd9562792
 
 # Compiled with Coconut version 1.5.0-post_dev57 [Fish License]
 
@@ -39,7 +39,7 @@ _coconut_sys.path.pop(0)
 
 # Installation constants:
 name = "bbopt"
-version = "1.3.1"
+version = "1.3.2"
 description = "The easiest hyperparameter optimization you'll ever do."
 long_description = """
 See BBopt's GitHub_ for more information.
@@ -56,7 +56,7 @@ extra_requirements["dev"] = (extra_requirements["examples"] + ("coconut-develop"
 
 
 # Optimizer constants:
-default_alg = "tpe_or_gp"
+default_alg = "any_fast"
 default_meta_alg = "epsilon_max_greedy"
 
 default_protocol = 2
@@ -76,4 +76,8 @@ default_jobs = 4
 # Backend constants:
 default_fallback_backend = "random"
 erroring_backend_errs = (ValueError, TypeError)
+
+
+# Mixture constants:
 eps_greedy_explore_prob = 0.2
+safe_fallback_alg = "tree_structured_parzen_estimator"
